/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 * \brief This file contains API for RS485 2 Click driver.
 *
 * \addtogroup rs4852 RS485 2 Click Driver
 * @{
 */
// ----------------------------------------------------------------------------

#ifndef RS4852_H
#define RS4852_H

#include "drv_digital_out.h"
#include "drv_uart.h"

// -------------------------------------------------------------- PUBLIC MACROS 
/**
 * \defgroup macros Macros
 * \{
 */

/**
 * \defgroup map_mikrobus MikroBUS
 * \{
 */
#define RS4852_MAP_MIKROBUS( cfg, mikrobus ) \
  cfg.tx_pin  = MIKROBUS( mikrobus, MIKROBUS_TX ); \
  cfg.rx_pin  = MIKROBUS( mikrobus, MIKROBUS_RX ); \
  cfg.oe = MIKROBUS( mikrobus, MIKROBUS_CS ); \
  cfg.re = MIKROBUS( mikrobus, MIKROBUS_PWM );
/** \} */

/**
 * \defgroup error_code Error Code
 * \{
 */
#define RS4852_RETVAL  uint8_t

#define RS4852_OK           0x00
#define RS4852_INIT_ERROR   0xFF
/** \} */

/**
 * \defgroup pins_power_mode Pins - Power mode
 * \{
 */
#define RS4852_RX_DISABLE 0x01
#define RS4852_RX_ENABLE  0x00

#define RS4852_TX_DISABLE 0x00
#define RS4852_TX_ENABLE  0x01

#define RS4852_RX_PIN 0x00
#define RS4852_TX_PIN 0x01
/** \} */

/** \} */ // End group macro 
// --------------------------------------------------------------- PUBLIC TYPES
/**
 * \defgroup type Types
 * \{
 */

typedef  uart_data_t rs4852_data_t;

/**
 * @brief Click ctx object definition.
 */
typedef struct
{
    // Output pins 

    digital_out_t oe;
    digital_out_t re;
    
    // Modules 

    uart_t uart;

} rs4852_t;

/**
 * @brief Click configuration structure definition.
 */
typedef struct
{
    // Communication gpio pins 

    pin_name_t rx_pin;
    pin_name_t tx_pin;
    
    // Additional gpio pins 

    pin_name_t oe;
    pin_name_t re;

    // static variable 

    hal_uart_baudrate_t   baud_rate;  // Clock speed. 
    hal_uart_data_bits_t  data_bit;   // Data bits.
    hal_uart_parity_bit_t parity_bit; // Parity bit.  
    hal_uart_stop_bits_t  stop_bit;   // Stop bits.  

} rs4852_cfg_t;

/** \} */ // End types group
// ----------------------------------------------- PUBLIC FUNCTION DECLARATIONS

/**
 * \defgroup public_function Public function
 * \{
 */
 
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief Config Object Initialization function.
 *
 * @param cfg  Click configuration structure.
 *
 * @description This function initializes click configuration structure to init state.
 * @note All used pins will be set to unconnected state.
 */
void rs4852_cfg_setup ( rs4852_cfg_t *cfg );

/**
 * @brief Initialization function.
 * @param rs4852 Click object.
 * @param cfg Click configuration structure.
 * 
 * @description This function initializes all necessary pins and peripherals used for this click.
 */
RS4852_RETVAL rs4852_init ( rs4852_t *ctx, rs4852_cfg_t *cfg );

/**
 * @brief Click Default Configuration function.
 *
 * @param ctx  Click object.
 *
 * @description This function executes default configuration for Rs4852 click.
 */
void rs4852_default_cfg ( rs4852_t *ctx );

/**
 * @brief Generic multi write function.
 *
 * @param ctx          Click object.
 * @param data_buf     Input data buffer
 * @param len          Number of the bytes in the data buffer
 *
 * @description This function writes data.
 */
void rs4852_generic_multi_write ( rs4852_t *ctx, rs4852_data_t *data_buf,  uart_length_t len );

/**
 * @brief Generic multi read function.
 *
 * @param ctx        Click object.
 * @param data_buf   Output data buffer.
 * @param len        Number of the bytes in data buffer.
 *
 * @description This function reads data.
 */
void rs4852_generic_multi_read ( rs4852_t *ctx, rs4852_data_t *data_buf,  uart_length_t len );

/**
 * @brief Generic single read function.
 *
 * @param ctx        Click object.
 * @return one byte data
 *
 * @description This function read one byte data.
 */
rs4852_data_t rs4852_generic_single_read ( rs4852_t *ctx );

/**
 * @brief Generic single write function.
 *
 * @param ctx          Click object.
 * @param tx_data      Input data
 *
 * @description This function write one byte data.
 */
void rs4852_generic_single_write ( rs4852_t *ctx, rs4852_data_t tx_data );

/**
 * @brief Power mode controle pins
 *
 * @param ctx    Click object.
 * @param pin    rx pin contorl -> RE pin or tx pin control -> OE pin
 * @param state  Pin disable or enable
 *
 * @description This function allows OE and RE pins to be controlled.
 *              To send data, RX pin control must be disabled, TX pin control is enabled.
 */
void rs4852_power_mode ( rs4852_t *ctx, uint8_t pin, uint8_t state );

#ifdef __cplusplus
}
#endif
#endif  // _RS4852_H_

/** \} */ // End public_function group
/// \}    // End click Driver group  
/*! @} */
// ------------------------------------------------------------------------- END
